import { Test, TestingModule } from '@nestjs/testing';
import { getRepositoryToken } from '@nestjs/typeorm';
import { Project } from '../projects/project.entity';
import { ReposService } from './repos.service';
import { ConfigService } from '@nestjs/config';
import { rm, unlink } from 'fs/promises';
import { join } from 'path';

const workspacesRoot = '/Users/ivanli/Projects/fennec/workspaces';

describe('ReposService', () => {
  let service: ReposService;
  const repositoryMockFactory = jest.fn(() => ({
    findOneOrFail: jest.fn(
      (entity): Project => ({
        id: '1',
        // sshUrl: 'ssh://gitea@git.ivanli.cc:7018/ivan/test1.git',
        sshUrl: 'ssh://gitea@git.ivanli.cc:7018/Fennec/fennec-fe.git',
        name: 'test1',
        ...entity,
      }),
    ),
  }));
  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        ReposService,
        {
          provide: getRepositoryToken(Project),
          useFactory: repositoryMockFactory,
        },
        {
          provide: ConfigService,
          useValue: {
            get() {
              return workspacesRoot;
            },
          },
        },
      ],
    }).compile();

    service = module.get<ReposService>(ReposService);
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });
  describe('listLogs', () => {
    beforeEach(async () => {
      await rm(join(workspacesRoot, 'test1'), { recursive: true });
    });
    it('should be return logs', async () => {
      const result = await service.listLogs({ projectId: '1' });
      expect(result).toBeDefined();
    }, 20_000);
  });
  describe('listBranch', () => {
    it('should be return branches', async () => {
      const result = await service.listBranches({ projectId: '1' });
      expect(result).toBeDefined();
    }, 10_000);
  });
});
